//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "f_ConditionConfig.h"
#include "f_ConditionFilterList.h"
#include "f_ConditionCondition.h"
#include "f_ConditionMonitor.h"
#include "f_VD15xProxy.h"
#include <filter.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TConditionconfig *Conditionconfig;

#include "f_ConditionIFPWrap.h"

// I had problems to generate it at run time
// This a workaround and we will have problems if we enable
// nesting of conditional!
IFilterPreviewWrap   ifp(0);

//---------------------------------------------------------------------------
__fastcall TConditionConfig::TConditionConfig(TComponent* Owner, FilterActivation* vfa, const FilterFunctions *vff)
        : TForm(Owner)//, ifp(fa->ifp)
{
  ifpProxy = 0;
  fa = vfa;
  ff = vff;
  mfd = *reinterpret_cast<CFD**>(fa->filter_data);

  ifp.Reassign(fa->ifp);
  ifpOriginal = fa->ifp;
  MonitorPreviewDlg = 0;

#ifdef _DEBUG
  Caption = Caption + " (debug version)";
#endif
}
//---------------------------------------------------------------------------

__fastcall TConditionConfig::~TConditionConfig()
{
  if (ifpProxy)
    delete ifpProxy;
}
//---------------------------------------------------------------------------
void __fastcall TConditionConfig::ButtonCallback(bool fNewState)
{
  if (ifpProxy)
    ifpProxy->ButtonCallback(fNewState);
}
void __fastcall TConditionConfig::SampleCallback(VFBitmap *Bitmap, long lFrame, long lCount)
{
  if (ifpProxy)
    ifpProxy->SampleCallback(Bitmap, lFrame, lCount);
}
//---------------------------------------------------------------------------
void __fastcall TConditionConfig::FormActivate(TObject *Sender)
{
  if (!mfd->Filter) {
    TFilterListDlg* Dlg = new TFilterListDlg(this);
    Dlg->ShowModal();
    if ((Dlg->ModalResult != mrOk) || !Dlg->FilterList->Selected) {
      delete Dlg;
      ModalResult = mrCancel;
      PostMessage(Handle, WM_CLOSE, 0, 0);
      return;
    }
    mfd->Filter = reinterpret_cast<FilterDefinition*>(Dlg->FilterList->Selected->Data);

    // VD 1.5.x attach from filter
    {
      CFilterDefinitionHelper fdh(fd_conditional);
      fdh.Attach(mfd->Filter);
    }

  }

  AnsiString Temp = mfd->Filter->name;
  if (mfd->Filter->maker) {
    Temp += " by ";
    Temp += mfd->Filter->maker;
  } else {
    Temp += " (internal)";
  }

  FilterName->Caption = Temp;
  Description->Caption = mfd->Filter->desc;

  ConditionList->Items->Clear();
  CBranch* Branch = mfd->Branches;
  while (Branch)
  {
    TListItem* Item = ConditionList->Items->Add();
    Item->Data = Branch;
    if (Branch->IsElse)
      Item->Caption = "else";
    else if (Branch == mfd->Branches)
      Item->Caption = "if";
    else
      Item->Caption = "else if";
    UpdateListItem(Item);
    Branch = Branch->Next;
  }

  if (ConditionList->Items->Count) {
    ConditionList->Items->Item[0]->Selected = true;
    ConditionList->Items->Item[0]->Focused  = true;
    ConditionList->SetFocus();
  }

  UpdateControls();
  ifp.InitButton(PreviewButton->Handle);

#ifdef ENABLE_RESIZING
  if (BorderStyle == bsDialog)
    BorderStyle = bsSizeable;
#endif
//  ifpProxy = new IFilterPreviewProxy(fa->ifp, Handle, PreviewButton->Handle);
}
//---------------------------------------------------------------------------
void __fastcall TConditionConfig::ConditionListClick(TObject *Sender)
{
  ConditionList->Selected = ConditionList->ItemFocused;
  UpdateControls();
}
//---------------------------------------------------------------------------
void __fastcall TConditionConfig::PreviewButtonClick(TObject *Sender)
{
  ifp.Toggle(Handle);
}
//---------------------------------------------------------------------------

void __cdecl ButtonCallback(bool fNewState, void *pData)
{
  TConditionConfig* Dlg = (TConditionConfig*) pData;
  Dlg->ButtonCallback(fNewState);
}

void __cdecl SampleCallback(VFBitmap *Bitmap, long lFrame, long lCount, void *pData)
{
  TConditionConfig* Dlg = (TConditionConfig*) pData;
  Dlg->SampleCallback(Bitmap, lFrame, lCount);
}
//---------------------------------------------------------------------------
void __fastcall TConditionConfig::UpdateControls()
{
  TListItem* Selected = ConditionList->Selected;

  // Add is always on
  RemoveCondition->Enabled = Selected;
  ConfigCondition->Enabled = Selected && mfd->Filter->configProc;
  EditCondition->Enabled = Selected && !reinterpret_cast<CBranch*>(Selected->Data)->IsElse;
  UpCondition->Enabled = EditCondition->Enabled && (Selected->Index != 0);
  DownCondition->Enabled = EditCondition->Enabled && (Selected->Index !=
    (ConditionList->Items->Count -
    (reinterpret_cast<CBranch*>(ConditionList->Items->Item[0]->Data)->HasElse() ? 2 : 1)));
}
//---------------------------------------------------------------------------
void __fastcall TConditionConfig::UpdateListItem(TListItem* Item)
{
  CBranch* Branch = reinterpret_cast<CBranch*>(Item->Data);
  Item->SubItems->Clear();
  if (!Branch->IsElse) {
    AnsiString FT = static_cast<CIFBranch*>(Branch)->pFormula->GetFormulaText();
    {
      char* FTB = new char [FT.Length() + 1];
      strcpy(FTB, FT.c_str());
      while (1) {
        char *CR = strchr(FTB, '\n');
        if (!CR) break;
        *CR = ' ';
      }
      while (1) {
        char *LF = strchr(FTB, '\r');
        if (!LF) break;
        *LF = ' ';
      }
      Item->SubItems->Add(FTB);
      delete [] FTB;
    }
  } else
    Item->SubItems->Add("");
  if (mfd->Filter->stringProc) {
    char Buffer[512];
    Buffer[0] = 0;
    char* pBuffer = Buffer;

    // fake
    fa->filter = mfd->Filter;
    fa->filter_data = Branch->pInstance->pData;

    mfd->Filter->stringProc(fa, ff, Buffer);

    {
      // post process

      // leading spaces
      while (*pBuffer && (*pBuffer == ' ')) ++pBuffer;

      // parent.
      if ((*pBuffer == '(') && (pBuffer[strlen(pBuffer) - 1] == ')')) {
        ++pBuffer;
        while (*pBuffer && (*pBuffer == ' ')) ++pBuffer;

        pBuffer[strlen(pBuffer) - 1] = 0;
      }
    }

    // restore
    fa->filter      = mfd->OwnFilter;
    fa->filter_data = mfd->OwnFilterData;

    Item->SubItems->Add(pBuffer);
  }
}
//---------------------------------------------------------------------------
void __fastcall TConditionConfig::RebuildChain()
{
  int Count = ConditionList->Items->Count;
  if (!Count) {
    mfd->Branches = 0;
    return;
  }

  long NeedFlags = 0;
  for (int i = 0; i < Count - 1; ++i)
  {
    CBranch* Branch = reinterpret_cast<CBranch*>(ConditionList->Items->Item[i]->Data);
    CBranch* Next = reinterpret_cast<CBranch*>(ConditionList->Items->Item[i+1]->Data);
    if (!i && !Branch->IsElse)
      NeedFlags |= static_cast<CIFBranch*>(Branch)->pFormula->GetNeedFlags();
    if (!Next->IsElse)
      NeedFlags |= static_cast<CIFBranch*>(Next)->pFormula->GetNeedFlags();

    Branch->Next  = Next;
  }
  reinterpret_cast<CBranch*>(ConditionList->Items->Item[Count - 1]->Data)->Next = 0;
  mfd->Branches = reinterpret_cast<CBranch*>(ConditionList->Items->Item[0]->Data);
  mfd->Branches->NeedFlags = NeedFlags;
}

//---------------------------------------------------------------------------

void __fastcall TConditionConfig::AddConditionExecute(TObject *Sender)
{
  TConditionEditDlg* Dlg = new TConditionEditDlg(this);
  if (mfd->Branches && mfd->Branches->HasElse())
    Dlg->ElsePossible = false;

  if (Dlg->ShowModal() == mrOk)
  try {
    CBranch* Branch;
    if (Dlg->RbElse->Checked)
      Branch = new CELSEBranch(fa, ff);
    else
      Branch = new CIFBranch(fa, ff, Dlg->Formula, Dlg->FormulaText.c_str());

    // insert and build the chain
    TListItem* Item;
    {
      int Position = -1;
      if (!Branch->IsElse)
        if (ConditionList->Items->Count)
        {
          if (! ConditionList->Selected)
            ConditionList->Selected = ConditionList->Items->Item[ConditionList->Items->Count - 1];

          Position = ConditionList->Selected->Index;
          if (ConditionList->Selected->Caption != "else")
            ++Position;
        }

      if (Position > -1)
        Item = ConditionList->Items->Insert(Position);
      else
        Item = ConditionList->Items->Add();

      if (Branch->IsElse)
        Item->Caption = "else";
      else if (!Item->Index)
        Item->Caption = "if";
      else
        Item->Caption = "else if";

      Item->Data = Branch;
      UpdateListItem(Item);
      UpdateControls();
      RebuildChain();

      Item->Selected = true;
      Item->Focused  = true;
      Application->ProcessMessages();
    }

    ifp.UndoSystem();
    ifp.RedoSystem();

    if (mfd->Filter->configProc) {
      // fake
      fa->filter = mfd->Filter;
      fa->filter_data = Branch->pInstance->pData;

      if (ifpProxy)
        delete ifpProxy;
      ifpProxy = new IFilterPreviewProxy(ifpOriginal, Handle, PreviewButton->Handle);
      fa->ifp = ifpProxy;
      mfd->PreviewBranch = Branch;

      try {
        try {
          if (mfd->Filter->configProc(fa, ff, Handle)) {
            fa->filter      = mfd->OwnFilter;
            fa->filter_data = mfd->OwnFilterData;
            fa->ifp         = ifpOriginal;
            mfd->PreviewBranch = 0;

            RemoveCondition->Execute();
          } else {
            UpdateListItem(Item);
          }
        } catch (...)
        {
        }
      } __finally {
        fa->filter      = mfd->OwnFilter;
        fa->filter_data = mfd->OwnFilterData;
        fa->ifp         = ifpOriginal;
        mfd->PreviewBranch = 0;

        delete ifpProxy;
        ifpProxy = 0;
      }
    }
  } __finally {
    delete Dlg;
  }
  if (ConditionList->Items->Count)
    ConditionList->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::RemoveConditionExecute(TObject *Sender)
{
  TListItem* Item = ConditionList->Selected;
  if (!Item) return;

  ifp.UndoSystem();

  CBranch* Branch = reinterpret_cast<CBranch*>(Item->Data);
  Item->Delete();
  Branch->Next = 0;
  Branch->DeleteBranch(fa, ff);
  RebuildChain();
  ConditionList->Selected = ConditionList->ItemFocused;
  UpdateControls();

  ifp.RedoSystem();
  if (ConditionList->Items->Count)
    ConditionList->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::UpConditionExecute(TObject *Sender)
{
  if (!ConditionList->Selected) return;

  void* Data = ConditionList->Selected->Data;
  TListItem* Item = ConditionList->Items->Item[ConditionList->Selected->Index - 1];
  ConditionList->Selected->Data = Item->Data;
  Item->Data = Data;
  UpdateListItem(ConditionList->Selected);
  UpdateListItem(Item);
  RebuildChain();
  Item->Selected = true;
  Item->Focused  = true;
  Item->MakeVisible(false);

  ifp.RedoFrame();
  if (ConditionList->Items->Count)
    ConditionList->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::DownConditionExecute(TObject *Sender)
{
  if (!ConditionList->Selected) return;

  void* Data = ConditionList->Selected->Data;
  TListItem* Item = ConditionList->Items->Item[ConditionList->Selected->Index + 1];
  ConditionList->Selected->Data = Item->Data;
  Item->Data = Data;
  UpdateListItem(ConditionList->Selected);
  UpdateListItem(Item);
  RebuildChain();
  Item->Selected = true;
  Item->Focused  = true;
  Item->MakeVisible(false);

  ifp.RedoFrame();
  if (ConditionList->Items->Count)
    ConditionList->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::EditConditionExecute(TObject *Sender)
{
  if (!ConditionList->Selected) return;

  TListItem* Item = ConditionList->Selected;
  if (!Item) return;
  CBranch* Branch = reinterpret_cast<CBranch*>(Item->Data);
  if (Branch->IsElse) return;

  CIFBranch* pIfBranch = static_cast<CIFBranch*>(Branch);
  TConditionEditDlg* Dlg = new TConditionEditDlg(this, pIfBranch->pFormula->GetFormulaText());
  Dlg->ElsePossible = false;

  if (Dlg->ShowModal() == mrOk) {
    if (Dlg->FormulaText != pIfBranch->pFormula->GetFormulaText())
    {
      pIfBranch->pFormula->DeleteObject();
      pIfBranch->pFormula = new CFormulaObject(Dlg->Formula, Dlg->FormulaText.c_str());
      UpdateListItem(Item);
    } else {
      delete Dlg->Formula;
    }

    ifp.RedoFrame();
  }
  delete Dlg;
  if (ConditionList->Items->Count)
    ConditionList->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::ConfigConditionExecute(TObject *Sender)
{
  TListItem* Item = ConditionList->Selected;
  if (!Item) return;
  CBranch* Branch = reinterpret_cast<CBranch*>(Item->Data);

  if (mfd->Filter->configProc) {
    // fake
    fa->filter = mfd->Filter;
    fa->filter_data = Branch->pInstance->pData;

    if (ifpProxy)
      delete ifpProxy;
    ifpProxy = new IFilterPreviewProxy(ifpOriginal, Handle, PreviewButton->Handle);
    fa->ifp = ifpProxy;

    mfd->PreviewBranch = Branch;

    try {
      mfd->Filter->configProc(fa, ff, Handle);
    } __finally {
      fa->filter      = mfd->OwnFilter;
      fa->filter_data = mfd->OwnFilterData;
      fa->ifp         = ifpOriginal;

      delete ifpProxy;
      ifpProxy = 0;
      mfd->PreviewBranch = 0;
    }

    ifp.RedoSystem();
  }
  UpdateListItem(Item);
  if (ConditionList->Items->Count)
    ConditionList->SetFocus();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::ConditionListDblClick(TObject *Sender)
{
  if (!ConditionList->Selected) {
    AddCondition->Execute();
    return;
  }

  if (GetKeyState(VK_MENU) < 0) {
    if (EditCondition->Enabled)
      EditCondition->Execute();
  } else {
    if (ConfigCondition->Enabled)
      ConfigCondition->Execute();
    else if (EditCondition->Enabled)
      EditCondition->Execute();
  }
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::ConditionListSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
  UpdateControls();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::Button3Click(TObject *Sender)
{
  if (!MonitorPreviewDlg)
    MonitorPreviewDlg = new TMonitorDlg(this, mfd);
  if (MonitorPreviewDlg->Showing)
    MonitorPreviewDlg->Hide();
  else
    MonitorPreviewDlg->Show();
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::FormDestroy(TObject *Sender)
{
  if (MonitorPreviewDlg) delete MonitorPreviewDlg;
  MonitorPreviewDlg = 0;
}
//---------------------------------------------------------------------------

void __fastcall TConditionConfig::Button1Click(TObject *Sender)
{
  CBranch* Branch = mfd->Branches;
  int Length = 0;
  while (Branch) {
    if (!Branch->IsElse)
      Length += strlen(static_cast<CIFBranch*>(Branch)->pFormula->GetFormulaText());
    Length += 100;
    Branch = Branch->Next;
  }
  if (Length > 1900)
    MessageBox(Handle, "Your settings are too long.\nYou may have problems storing them for the batch job!", "Warning", MB_OK | MB_ICONWARNING);
}
//---------------------------------------------------------------------------

