//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------
//#pragma hdrstop
#pragma inline

#include "f_ConditionIFPWrap.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

#define FUN0(offset)                  \
  __asm {                             \
    mov ecx, [esp + 0x04] /* p */    ;\
    mov edx, [ecx]        /* vmt */  ;\
    call dword ptr [edx + offset]    ;\
    ret 0x0004                       ;\
  }

#define FUN1(offset)                  \
  __asm {                             \
    mov ecx, [esp + 0x04] /* p */    ;\
    mov eax, [esp + 0x08] /* p1 */   ;\
    mov edx, [ecx]        /* vmt */  ;\
    push eax                         ;\
    call dword ptr [edx + offset]    ;\
    ret 0x0008                       ;\
  }

#define FUN2(offset)                  \
  __asm {                             \
    push ebp                         ;\
    mov ebp, esp                     ;\
    mov ecx, [ebp + 0x08] /* p */    ;\
    mov edx, [ecx]        /* vmt */  ;\
    mov eax, [ebp + 0x10] /* p2 */   ;\
    push eax                         ;\
    mov eax, [ebp + 0x0c] /* p1 */   ;\
    push eax                         ;\
    call dword ptr [edx + offset]    ;\
    pop ebp                          ;\
    ret 0x000c                       ;\
  }

#define FUN3(offset)                  \
  __asm {                             \
    push ebp                         ;\
    mov ebp, esp                     ;\
    mov ecx, [ebp + 0x08] /* p */    ;\
    mov edx, [ecx]        /* vmt */  ;\
    mov eax, [ebp + 0x14] /* p3 */   ;\
    push eax                         ;\
    mov eax, [ebp + 0x10] /* p2 */   ;\
    push eax                         ;\
    mov eax, [ebp + 0x0c] /* p1 */   ;\
    push eax                         ;\
    call dword ptr [edx + offset]    ;\
    pop ebp                          ;\
    ret 0x0010                       ;\
  }

#pragma warn -8070 // no return
__declspec(naked)
void __stdcall ifp_SetButtonCallback (void* p, FilterPreviewButtonCallback cb, void *d)
{ FUN2(0); }
__declspec(naked)
void __stdcall ifp_SetSampleCallback (void* p, FilterPreviewSampleCallback cb, void *d)
{ FUN2(0x04); }
__declspec(naked)
long __stdcall ifp_isPreviewEnabled (void* p)
{ FUN0(0x08); }
__declspec(naked)
void __stdcall ifp_Toggle (void* p, HWND w)
{ FUN1(0x0c); }
__declspec(naked)
void __stdcall ifp_Display (void* p, HWND w, long d)
{ FUN2(0x10); }
__declspec(naked)
void __stdcall ifp_RedoFrame (void* p)
{ FUN0(0x14); }
__declspec(naked)
void __stdcall ifp_RedoSystem (void* p)
{ FUN0(0x18); }
__declspec(naked)
void __stdcall ifp_UndoSystem (void* p)
{ FUN0(0x1c); }
__declspec(naked)
void __stdcall ifp_InitButton (void* p, HWND)
{ FUN1(0x20); }
__declspec(naked)
void __stdcall ifp_Close (void* p)
{ FUN0(0x24); }
__declspec(naked)
long __stdcall ifp_SampleCurrentFrame (void* p)
{ FUN0(0x28); }
__declspec(naked)
long __stdcall ifp_SampleFrames (void* p)
{ FUN0(0x2c); }
#pragma warn .8070 // no return

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
IFilterPreviewWrap::IFilterPreviewWrap(IFilterPreview* vifp)
{
  ifp = vifp;
}
void IFilterPreviewWrap::Reassign(IFilterPreview* vifp)
{
  ifp = vifp;
}

void IFilterPreviewWrap::SetButtonCallback(FilterPreviewButtonCallback cb, void *d)
{ ifp_SetButtonCallback(ifp, cb, d); }
void IFilterPreviewWrap::SetSampleCallback(FilterPreviewSampleCallback cb, void *d)
{ ifp_SetSampleCallback(ifp, cb, d); }
bool IFilterPreviewWrap::isPreviewEnabled()
{ return ifp_isPreviewEnabled(ifp); }
void IFilterPreviewWrap::Toggle(HWND w)
{ ifp_Toggle(ifp, w); }
void IFilterPreviewWrap::Display(HWND w, bool d)
{ ifp_Display(ifp, w, d); }
void IFilterPreviewWrap::RedoFrame()
{ ifp_RedoFrame(ifp); }
void IFilterPreviewWrap::RedoSystem()
{ ifp_RedoSystem(ifp); }
void IFilterPreviewWrap::UndoSystem()
{ ifp_UndoSystem(ifp); }
void IFilterPreviewWrap::InitButton(HWND w)
{ ifp_InitButton(ifp, w); }
void IFilterPreviewWrap::Close()
{ ifp_Close(ifp); }
bool IFilterPreviewWrap::SampleCurrentFrame()
{ return ifp_SampleCurrentFrame(ifp); }
long IFilterPreviewWrap::SampleFrames()
{ return ifp_SampleFrames(ifp); }

#define PROXYJUMP(offset)       \
  __asm {                       \
    pop edx   /* pop ret */    ;\
    push ecx  /* push this */  ;\
    push edx  /* push ret */   ;\
    mov  edx, [ecx] /* vmt */  ;\
    push [edx + 0x30 + offset] ;\
    ret                        ;\
  }

#pragma warn -8070 // no return
__declspec(naked)
void IFilterPreviewProxy::SetButtonCallback(FilterPreviewButtonCallback cb, void *d)
{ PROXYJUMP(0x00); }
__declspec(naked)
void IFilterPreviewProxy::SetSampleCallback(FilterPreviewSampleCallback cb, void *d)
{ PROXYJUMP(0x04); }
__declspec(naked)
bool IFilterPreviewProxy::isPreviewEnabled()
{ PROXYJUMP(0x08); }
__declspec(naked)
void IFilterPreviewProxy::Toggle(HWND w)
{ PROXYJUMP(0x0c); }
__declspec(naked)
void IFilterPreviewProxy::Display(HWND w, bool d)
{ PROXYJUMP(0x10); }
__declspec(naked)
void IFilterPreviewProxy::RedoFrame()
{ PROXYJUMP(0x14); }
__declspec(naked)
void IFilterPreviewProxy::RedoSystem()
{ PROXYJUMP(0x18); }
__declspec(naked)
void IFilterPreviewProxy::UndoSystem()
{ PROXYJUMP(0x1c); }
__declspec(naked)
void IFilterPreviewProxy::InitButton(HWND w)
{ PROXYJUMP(0x20); }
__declspec(naked)
void IFilterPreviewProxy::Close()
{ PROXYJUMP(0x24); }
__declspec(naked)
bool IFilterPreviewProxy::SampleCurrentFrame()
{ PROXYJUMP(0x28); }
__declspec(naked)
long IFilterPreviewProxy::SampleFrames()
{ PROXYJUMP(0x2c); }
#pragma warn .8070 // no return

IFilterPreviewProxy::IFilterPreviewProxy(IFilterPreview* vifp, HWND DialogHandle, HWND ButtonHandle)
  : ifp(vifp)
{
  hButton = ButtonHandle;
  hDialog = DialogHandle;
  pButtonCB      = 0;
  pvButtonCBData = 0;
  pSampleCB      = 0;
  pvSampleCBData = 0;

  // force to compile all function
  int a = 10;
  if (a-- == a*a) {
    isPreviewEnabled();
    Toggle(0);
    Display(0, false);
    RedoFrame();
    RedoSystem();
    UndoSystem();
    InitButton(0);
    Close();
    SampleCurrentFrame();
    SampleFrames();
    SampleFrames();
    _isPreviewEnabled();
    _Toggle(0);
    _Display(0, false);
    _RedoFrame();
    _RedoSystem();
    _UndoSystem();
    _InitButton(0);
    _Close();
    _SampleCurrentFrame();
    _SampleFrames();
    _SampleFrames();
  }
}

void IFilterPreviewProxy::ButtonCallback (bool fNewState)
{
  if (hButton)
    SetWindowText(hButton, fNewState ? "Hide preview" : "Show preview");

  if (pButtonCB)
    pButtonCB(fNewState, pvButtonCBData);
}

void IFilterPreviewProxy::SampleCallback (VFBitmap *pBitmap, long lFrame, long lCount)
{
  if (pSampleCB)
    pSampleCB(pBitmap, lFrame, lCount, pvSampleCBData);
}
extern void dout(const char*,...);
void __stdcall IFilterPreviewProxy::_SetButtonCallback(FilterPreviewButtonCallback cb, void *d)
{ dout(__FUNC__);
  pButtonCB = cb;
  pvButtonCBData = d;
}
void __stdcall IFilterPreviewProxy::_SetSampleCallback(FilterPreviewSampleCallback cb, void *d)
{ dout(__FUNC__);
  pSampleCB = cb;
  pvSampleCBData = d;
}
bool __stdcall IFilterPreviewProxy::_isPreviewEnabled()
{ dout(__FUNC__);
  return ifp.isPreviewEnabled();
}
void __stdcall IFilterPreviewProxy::_Toggle(HWND w)
{ dout(__FUNC__);
  ifp.Toggle(hDialog);
}
void __stdcall IFilterPreviewProxy::_Display(HWND w, bool d)
{ dout(__FUNC__);
  ifp.Display(hDialog, d);
}
void __stdcall IFilterPreviewProxy::_RedoFrame()
{ dout(__FUNC__);
  ifp.RedoFrame();
}
void __stdcall IFilterPreviewProxy::_RedoSystem()
{ dout(__FUNC__);
  ifp.RedoSystem();
}
void __stdcall IFilterPreviewProxy::_UndoSystem()
{ dout(__FUNC__);
  ifp.UndoSystem();
}
void __stdcall IFilterPreviewProxy::_InitButton(HWND w)
{ dout(__FUNC__);

  char Buffer[32];
  bool Visible = false;
  if (hButton && GetWindowText(hButton, Buffer, sizeof(Buffer)))
    Visible = !stricmp("Hide preview", Buffer);

  hButton = w;
  if (hButton) {
    if (ifp.isPreviewEnabled()) {
      EnableWindow(hButton, TRUE);
      SetWindowText(hButton, Visible ? "Hide preview" : "Show preview");
    } else {
      EnableWindow(hButton, FALSE);
    }
  }
}
void __stdcall IFilterPreviewProxy::_Close()
{ dout(__FUNC__);
  // do nothing
  // this function is called usually at dialog end
}
bool __stdcall IFilterPreviewProxy::_SampleCurrentFrame()
{ dout(__FUNC__);
  return ifp.SampleCurrentFrame();
}
long __stdcall IFilterPreviewProxy::_SampleFrames()
{ dout(__FUNC__);
  return ifp.SampleFrames();
}

