//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------

#ifndef f_ConditionIFPWrapH
#define f_ConditionIFPWrapH
//---------------------------------------------------------------------------

#include <filter.h>

//
// Wrap to use IFilterPreview class in Borland Builder
//
// I was unable to tweak the Borland Builder to produce object calls
// with the same call convention as Microsoft Visual C++.
// Therefore I wrote this wrapper class which remaps all calls
// in the way the VirtualDub objects acept them.
//
class IFilterPreviewWrap {
  private:
    IFilterPreview* ifp;

  public:
    IFilterPreviewWrap(IFilterPreview* vifp);
    void Reassign(IFilterPreview* vifp);

  public:
    void SetButtonCallback(FilterPreviewButtonCallback, void *);
    void SetSampleCallback(FilterPreviewSampleCallback, void *);

    bool isPreviewEnabled();
    void Toggle(HWND);
    void Display(HWND, bool);
    void RedoFrame();
    void RedoSystem();
    void UndoSystem();
    void InitButton(HWND);
    void Close();
    bool SampleCurrentFrame();
    long SampleFrames();
};

//
// Proxy to offer IFilterPreview class in other filter modules
//
// dito as before. If we want to pass our class as proxy to other
// filters, we have to offer the same call convetion as VirtualDub.
//
class IFilterPreviewProxy: public IFilterPreview {
  private:
    IFilterPreviewWrap ifp;
    HWND            hButton;
    HWND            hDialog;
    FilterPreviewButtonCallback pButtonCB;
    void*                       pvButtonCBData;
    FilterPreviewSampleCallback pSampleCB;
    void*                       pvSampleCBData;

  public:
    void SetButtonCallback(FilterPreviewButtonCallback cb, void *d);
    void SetSampleCallback(FilterPreviewSampleCallback cb, void *d);

    bool isPreviewEnabled();
    void Toggle(HWND w);
    void Display(HWND w, bool d);
    void RedoFrame();
    void RedoSystem();
    void UndoSystem();
    void InitButton(HWND w);
    void Close();
    bool SampleCurrentFrame();
    long SampleFrames();

  public:
    IFilterPreviewProxy(IFilterPreview* vifp, HWND DialogHandle, HWND ButtonHandle);

  public:
    void ButtonCallback (bool fNewState);
    void SampleCallback (VFBitmap *, long lFrame, long lCount);

  private:
    virtual void __stdcall _SetButtonCallback(FilterPreviewButtonCallback cb, void *d);
    virtual void __stdcall _SetSampleCallback(FilterPreviewSampleCallback cb, void *d);

    virtual bool __stdcall _isPreviewEnabled();
    virtual void __stdcall _Toggle(HWND w);
    virtual void __stdcall _Display(HWND w, bool d);
    virtual void __stdcall _RedoFrame();
    virtual void __stdcall _RedoSystem();
    virtual void __stdcall _UndoSystem();
    virtual void __stdcall _InitButton(HWND w);
    virtual void __stdcall _Close();
    virtual bool __stdcall _SampleCurrentFrame();
    virtual long __stdcall _SampleFrames();
};

#endif

