//---------------------------------------------------------------------------
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 1.0     Oktober 2001
//      Version 2.0     March 2003
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "f_ConditionMonitor.h"
#include "f_ConditionMonitorValue.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "PERFGRAP"
#pragma resource "*.dfm"
//TMonitorDlg *MonitorDlg;

#include "f_ConditionLogic.h"

class CMonDlgEntry {
  public:
    AnsiString Name;
    TColor     Color;
    CNumeric*  Numeric;
};

//---------------------------------------------------------------------------
__fastcall TMonitorDlg::TMonitorDlg(TComponent* Owner, CFD* vmfd)
        : TForm(Owner)
{
  pMonitorValues = 0;
  mfd = vmfd;
  PGCounter = 0;
}
//---------------------------------------------------------------------------

void __fastcall TMonitorDlg::Button1Click(TObject *Sender)
{
  int StepSize = PG->StepSize;
  PG->StepSize = PG->Width;
  PG->DataPoint(clBlack, 0); PG->Update();
  PG->DataPoint(clBlack, 0); PG->Update();
  PG->DataPoint(clBlack, 0); PG->Update();
  PG->DataPoint(clBlack, 0); PG->Update();
  PG->DataPoint(clBlack, 0); PG->Update();
  PG->StepSize = StepSize;
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDlg::FormActivate(TObject *Sender)
{
  mfd->MonitorPreview = &pMonitorValues;
  mfd->MonitorPreviewHandle = Handle;
  for (int i=0; i < MonitorList->Items->Count; ++i)
  if (!MonitorList->Items->Item[i]->Data)
  {
    // initialize the standard entries
    // will be removed next time
    CMonDlgEntry* Data =  new CMonDlgEntry();
    Data->Name = MonitorList->Items->Item[i]->Caption;
    switch (i) {
      case 0: Data->Numeric = new NFrame();
              Data->Color = clBlack; break;
      case 1: Data->Numeric = new NStamp();
              Data->Color = clBlack; break;
      default: Data->Numeric = new NLuma();
               Data->Color = clRed; break;
    }

    MonitorList->Items->Item[i]->Data = Data;
  }
  MakeChain();

  {
    ExternalList->Items->Clear();
    for (int i = 0; i < 100; ++i) {
      if (! ExternalFunctions[i].name) break;

      TListItem* Item = ExternalList->Items->Add();
      Item->Caption = ExternalFunctions[i].name;
      Item->SubItems->Add(ExternalFunctions[i].maker);
      Item->SubItems->Add(ExternalFunctions[i].description);
    }
  }

#ifdef ENABLE_RESIZING
  if (BorderStyle == bsDialog)
    BorderStyle = bsSizeable;
#endif
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDlg::FormDestroy(TObject *Sender)
{
  pMonitorValues = 0;
  mfd->MonitorPreview = 0;
  mfd->MonitorPreviewHandle = 0;
  for (int i=0; i < MonitorList->Items->Count; ++i)
  {
    CMonDlgEntry* Data = reinterpret_cast<CMonDlgEntry*>(MonitorList->Items->Item[i]->Data);
    delete Data->Numeric;
    delete Data;
  }
}
//---------------------------------------------------------------------------
void TMonitorDlg::MakeChain()
{
  pMonitorValues = 0;
  CMonDlgEntry* Last = 0, *First = 0;
  for (int i=0; i < MonitorList->Items->Count; ++i)
  {
    CMonDlgEntry* Data = reinterpret_cast<CMonDlgEntry*>(MonitorList->Items->Item[i]->Data);
    if (!Last) {
      Last = First = Data;
    } else {
      Last->Numeric->Next = Data->Numeric;
      Last = Data;
    }
  }
  if (Last) Last->Numeric->Next = 0;
  if (First)
    pMonitorValues = First->Numeric;
}
//---------------------------------------------------------------------------
void TMonitorDlg::MakeItem(TListItem* Item)
{
  TMonitorValueDlg* Dlg = new TMonitorValueDlg(this);
  CMonDlgEntry* Data = 0;
  if (Item) {
    Data = reinterpret_cast<CMonDlgEntry*>(Item->Data);
    Dlg->NumericText = Data->Name;
    Dlg->NumericColor = Data->Color;
  } else {
    Dlg->NumericText = "";
  }
  switch (Dlg->ShowModal()) {
    case mrOk:
      if (Data) {
        Data->Name = Dlg->NumericText;
        Data->Color = Dlg->NumericColor;
        CNumeric* Numeric = Data->Numeric;
        Data->Numeric = Dlg->Numeric;
        MakeChain();
        Item->SubItems->Clear();
        Sleep(0);
        delete Numeric;
      } else {
        Data = new CMonDlgEntry();
        Data->Name = Dlg->NumericText;
        Data->Color = Dlg->NumericColor;
        Data->Numeric = Dlg->Numeric;
        Item = MonitorList->Items->Add();
        Item->Data = Data;
        MakeChain();
      }
      Item->Caption = Data->Name;
      break;
    case mrAbort:
      if (Data) {
        Item->Delete();
        MakeChain();
        Sleep(0);
        delete Data->Numeric;
        delete Data;
      } break;
  }
  delete Dlg;
}
void __fastcall TMonitorDlg::MonitorListDblClick(TObject *Sender)
{
  MakeItem(MonitorList->Selected);
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDlg::MonitorListChange(TObject *Sender,
      TListItem *Item, TItemChange Change)
{
  MonitorList->Selected = MonitorList->ItemFocused;
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDlg::ExternalListChange(TObject *Sender,
      TListItem *Item, TItemChange Change)
{
  ExternalList->Selected = ExternalList->ItemFocused;
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDlg::WMAddValue(TMessage &Message)
{
dout(__FUNC__);
  if (Showing) {
    int Index = Message.WParam;
    int Value = Message.LParam;
    if ((Index >= 0) && (Index < MonitorList->Items->Count)) {
      TListItem *Item = MonitorList->Items->Item[Index];
      Item->SubItems->Clear();
      Item->SubItems->Add(IntToStr(Value));
      CMonDlgEntry* Data = reinterpret_cast<CMonDlgEntry*>(Item->Data);
      if (Data->Color != clBlack) {
        PG->DataPoint(Data->Color, Value);
        ++PGCounter;
      }
    }
  }
  Message.Result = 1;
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDlg::WMUpdate(TMessage &Message)
{
dout(__FUNC__);
  if (Showing && PGCounter) {
    PG->Update();
    PGCounter = 0;
  }
  Message.Result = 1;
}
void __fastcall TMonitorDlg::WMCondition(TMessage &Message)
{
dout(__FUNC__);
  if (Showing) {
    switch (Message.LParam)
    {
      case -2: ConditionLabel->Caption = "--"; break;
      case -1: ConditionLabel->Caption = "no one"; break;
      default: ConditionLabel->Caption = IntToStr(Message.LParam);
    }
  }
  Message.Result = 1;
}
//---------------------------------------------------------------------------
void __fastcall TMonitorDlg::Button2Click(TObject *Sender)
{
  pMonitorValues = 0;
  Close();
}
//---------------------------------------------------------------------------

void __fastcall TMonitorDlg::Button3Click(TObject *Sender)
{
  MakeItem(0);
}
//---------------------------------------------------------------------------

