//---------------------------------------------------------------------------
//      The original file was modified for the following application:
//
//	Conditional Filter for VirtualDub - implements simple branching
//	Copyright (C) 2001-2003 Dmitri Schamschurko
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//      The author can be contacted at:
//      Dmitri Schamschurko
//      dc1@tcs.inf.tu-dresden.de
//
//      Version 2.0     March 2003
//---------------------------------------------------------------------------
//
// All inline methods with asm instructions are replaced
// by conventional call with implementation in atomicint.cpp
//
//---------------------------------------------------------------------------
#include <vd2/system/atomic.h>


        int VDAtomicInt::staticExchange(volatile int *dst, int v) {
/*
		__asm {
			mov eax,v
			mov edx,dst
			lock xchg [dst],eax
			mov v,eax
		}
		return v;
*/
                int t = *dst;
                *dst  = v;
                return t;
	}

	void VDAtomicInt::staticIncrement(volatile int *dst) {
/*
		__asm {
			mov eax,dst
			lock inc dword ptr [eax]
		}
*/
                ++*dst;
	}

	void VDAtomicInt::staticDecrement(volatile int *dst) {
/*
		__asm {
			mov eax,dst
			lock dec dword ptr [eax]
		}
*/
                --*dst;
	}

	bool VDAtomicInt::staticDecrementTestZero(volatile int *dst) {
/*
		bool b;

		__asm {
			mov eax,dst
			lock dec dword ptr [eax]
			setz al
			mov b,al
		}
*/
                --*dst;
                return *dst == 0;
	}

	void VDAtomicInt::staticAdd(volatile int *dst, int v) {
/*
		__asm {
			mov eax,v
			mov	edx,dst
			lock add [edx],eax
		}
*/
                *dst += v;
	}

/*
	int VDAtomicInt::staticAddResult(volatile int *dst, int v) {
		__asm {
			mov eax,v
			mov	edx,dst
			lock xadd [edx],eax
			add	v,eax
		}
		return v;
	}
*/

	///////////////////////////////

	void VDAtomicInt::operator++() {
/*
		__asm mov ecx,this
		__asm lock inc dword ptr [ecx]
*/
                ++n;
	}

	void VDAtomicInt::operator--() {
/*
		__asm mov ecx,this
		__asm lock dec dword ptr [ecx]
*/
                --n;
	}


	void VDAtomicInt::operator+=(int v) {
/*
		__asm mov eax,v
		__asm mov ecx,this
		__asm lock add dword ptr [ecx],eax
*/
                n += v;
	}

	void VDAtomicInt::operator-=(int v) {
/*
		__asm mov eax,v
		__asm mov ecx,this
		__asm lock sub dword ptr [ecx],eax
*/
                n -= v;
	}


	int VDAtomicInt::xchg(int v) {
/*
		__asm mov eax,v
		__asm mov ecx,this
		__asm xchg dword ptr [ecx], eax
*/
                int t = n;
                n = v;
                return t;
	}

/*
	bool VDAtomicInt::inctestzero() {
		__asm mov ecx,this
		__asm lock inc dword ptr [ecx]
		__asm setz al
	}
*/

	bool VDAtomicInt::dectestzero() {
/*
		__asm mov ecx,this
		__asm lock dec dword ptr [ecx]
		__asm setz al
*/
                --n;
                return n == 0;
	}

/*
	bool VDAtomicInt::addcarry(int v) {
		__asm mov eax,v
		__asm mov ecx,this
		__asm lock add dword ptr [ecx], eax
		__asm sbb eax,eax
		__asm and eax,1
	}

	bool VDAtomicInt::subcarry(int v) {
		__asm mov eax,v
		__asm mov ecx,this
		__asm lock sub dword ptr [ecx], eax
		__asm sbb eax,eax
		__asm and eax,1
	}
*/
	// 486 only, but much nicer.  They return the actual result.

/*
	int VDAtomicInt::inc() {
		__asm mov eax,1
		__asm mov ecx,this
		__asm lock xadd dword ptr [ecx],eax
		__asm inc eax
	}
*/
	int VDAtomicInt::dec() {
/*
		__asm mov eax,-1
		__asm mov ecx,this
		__asm lock xadd dword ptr [ecx],eax
		__asm dec eax
*/
                --n;
                return n;
	}

/*
	int VDAtomicInt::add(int v) {
		__asm mov eax,v
		__asm mov ecx,this
		__asm lock xadd dword ptr [ecx],eax
		__asm add eax,v
	}

	// These return the result before the operation, which is more inline with
	// what XADD allows us to do.
*/

	int VDAtomicInt::postinc() {
/*
		__asm mov eax,1
		__asm mov ecx,this
		__asm lock xadd dword ptr [ecx],eax
*/
                int t = n;
                ++n;
                return t;
	}

/*
	int VDAtomicInt::postdec() {
		__asm mov eax,-1
		__asm mov ecx,this
		__asm lock xadd dword ptr [ecx],eax
	}

	int VDAtomicInt::postadd(int v) {
		__asm mov eax,v
		__asm mov ecx,this
		__asm lock xadd dword ptr [ecx],eax
	}

*/
